;------------------------------------------------------------------
;
;      HelloWorld3 - copyright Jeremy Gordon 2002-2003
;
;      "HELLO WORLD" WINDOWS GDI PROGRAM - demonstrating using
;      formalised structures, automated stack frames using FRAME..ENDF
;      USEDATA and USES, INVOKE, definitions and named re-usable labels 
;      - in GoAsm syntax
;
;      Assemble using GoAsm HelloWorld3 (produces PE COFF file)
;      Then link using:-
;      GoLink HelloWorld3.obj user32.dll kernel32.dll gdi32.dll
;      (add -debug coff to run through the program using the debugger)
;
;------------------------------------------------------------------
;
;
;This has been radically altered by me to suit my needs
;The output and program flow is the exact same though
;Just the program architecture has changed
;Donkey :)
;
;First lets specify a couple of formalised structures
#include "\RadASM\GoAsm\inc\RadDbg.a"

WNDCLASSEX STRUCT
	cbSize			DD 30h	;+0 size of the structure (WNDCLASSEX only)
	style			DD 		;+4 window class style (CS_)
	lpfnWndProc		DD 		;+8 pointer to Window Procedure
	cbClsExtra		DD 		;+C no. of extra bytes to allocate after structure
	cbWndExtra		DD 		;+10 no. of extra bytes to allocate after window instance
	hInstance		DD 		;+14 handle to instance for this window class
	hIcon			DD 		;+18 handle to the class icon
	hCursor			DD 		;+1C handle to the class cursor
	hbrBackground	DD 		;+20 identifies the class background brush
	lpszMenuName	DD 		;+24 pointer to resource name for class menu
	lpszClassName	DD 		;+28 pointer to string for window class name
	hIconSm			DD 		;+2C handle to the small icon
ENDS

RECT STRUCT
      left			DD 
      top			DD 
      right			DD 
      bottom		DD 
ENDS

PAINTSTRUCT STRUCT
	hdc				DD 
	fErase			DD 
	rcPaint			RECT
	fRestore		DD 
	fIncUpdate		DD 
	rgbReserved		DB 32 DUP 
ENDS

;##################################################################

DATA SECTION
	wcex				WNDCLASSEX	;establish the WNDCLASS structure in data
	hInst				DD 0		;handle to the process itself
	WINDOW_CLASSNAME	DB 'WC',0	;string to hold name of window class
	MSG			DD 7 DUP 0			;structure to hold messages from Windows as follows:-
									;hWnd, +4=message, +8=wParam, +C=lParam, +10h=time, +14h/18=pt

;##################################################################

.CONST
	;and now some definitions for later use

	;*** first some window class styles
	CS_VREDRAW		equ		1h
	CS_HREDRAW		equ		2h
	CS_CLASSDC		equ		40h

	;*** now some window styles
	WS_POPUP		equ		80000000h
	WS_VISIBLE		equ		10000000h
	WS_CAPTION		equ		0C00000h
	WS_SIZEBOX		equ		40000h
	WS_SYSMENU		equ		80000h
	WS_MINIMIZEBOX	equ		20000h
	WS_MAXIMIZEBOX	equ		10000h
	WM_COPYDATA		equ		4Ah
	;*** messages we shall be dealing with
	WM_CREATE		equ		1h
	WM_DESTROY		equ		2h
	WM_PAINT		equ		0Fh

	;*** and miscellaneous
	COLOR_WINDOW	equ		5


CODE SECTION

INITIALISE_WNDCLASS:    ;get ready for window
	MOV EBX,ADDR wcex
	ADD EBX,4               ;jump over size dword
	MOV EAX,10
	L1:
		MOV D[EBX+EAX*4],0      ;fill rest of structure with zeroes
		DEC EAX
	JNS L1
	
	;***** add things to window class for all windows in the program ..
	MOV EAX,[hInst]         ;get handle to the process
	MOV [wcex.hInstance],EAX   ;give to window class structure
	MOV D[wcex.hbrBackground],COLOR_WINDOW+1  ;set background brush
	RET

;##################################################################
;
CODE SECTION

Start:
	INVOKE GetModuleHandleA, 0			;get handle to the process
	MOV [hInst],EAX						;record it in data label
	CALL INITIALISE_WNDCLASS			;get ready to register window class

;################# now add things specific to the window to be made

	MOV D[wcex.style],CS_VREDRAW+CS_HREDRAW+CS_CLASSDC
	MOV [wcex.lpfnWndProc],ADDR WndProc						;window procedure
	MOV [wcex.lpszClassName],ADDR WINDOW_CLASSNAME			;window class name

	INVOKE RegisterClassExA,ADDR wcex						;register the window class
	INVOKE CreateWindowExA, 0, ADDR WINDOW_CLASSNAME,	\
		'Hello World window made by GoAsm',				\	;title string
		WS_POPUP|WS_VISIBLE|WS_CAPTION|WS_SIZEBOX		\	;window style
		|WS_SYSMENU|WS_MINIMIZEBOX|WS_MAXIMIZEBOX,		\
		50,50,											\	;x-pos then y-pos
		320,200,										\	;width then height
		0,0,[hInst],0

;################# now enter the main message loop
;
.messloop
	INVOKE GetMessageA, ADDR MSG,0,0,0
	OR EAX,EAX								;see if it is WM_QUIT (eax=0)
	JZ >.quit								;yes
		INVOKE TranslateMessage, ADDR MSG
		INVOKE DispatchMessageA, ADDR MSG
JMP .messloop								;after message dealt with, loop back for next one

.quit
	INVOKE UnregisterClassA, ADDR WINDOW_CLASSNAME,[hInst]
	INVOKE ExitProcess, [MSG+8h]

WndProc FRAME hwnd,uMsg,wParam,lParam		;establish stack frame and get parameters
	LOCAL lpPaint	:PAINTSTRUCT
	LOCAL hDC		:D

	MOV EAX,[uMsg]							;get in eax message sent by Windows

	.WMCREATE
	CMP EAX,WM_CREATE
		JNE >.WMDESTROY
		JMP >.exit

	.WMDESTROY
	CMP EAX,WM_DESTROY
		JNE >.WMPAINT
		PUSH 0
		CALL PostQuitMessage    			;exit via the message loop
		JMP >.DEFPROC

	.WMPAINT
	CMP EAX,WM_PAINT
		JNE >.DEFPROC
		INVOKE BeginPaint, [hwnd],ADDR lpPaint     ;get in eax the DC to use
		MOV [hDC],EAX                              ;save it in local data

		INVOKE Ellipse,	[hDC],	[lpPaint.rcPaint.left],		\
								[lpPaint.rcPaint.top] ,		\
								[lpPaint.rcPaint.right],	\
								[lpPaint.rcPaint.bottom]

		INVOKE EndPaint, [hwnd],ADDR lpPaint		;give DC back to Windows
		JMP >.exit

	.DEFPROC
		INVOKE DefWindowProcA,[hwnd],[uMsg],[wParam],[lParam]
		RET

	.exit

	XOR EAX,EAX
	RET
WndProc ENDF								;finish this stack frame
